@@ -1,27 +1,24 @@
 /*
  *  Copyright 2013-2014  Andreas Cord-Landwehr <cordlandwehr@kde.org>
  *
- *  This program is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU General Public License as
- *  published by the Free Software Foundation; either version 2 of
- *  the License or (at your option) version 3 or any later version
- *  accepted by the membership of KDE e.V. (or its successor approved
- *  by the membership of KDE e.V.), which shall act as a proxy
- *  defined in Section 14 of version 3 of the license.
+ *  This library is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as published
+ *  by the Free Software Foundation; either version 2.1 of the License, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  GNU Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
+ *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "outputdevicecontroller.h"
 #include "qtgstreameroutputbackend.h"
-#include <KUrl>
-#include <KDebug>
+#include <QUrl>
+#include "libsound_debug.h"
 
 /**
  * \class OutputDeviceControllerPrivate
@@ -96,7 +93,7 @@ void OutputDeviceController::play(const QString& filePath)
     emit started();
 }
 
-void OutputDeviceController::play(const KUrl &filePath)
+void OutputDeviceController::play(const QUrl &filePath)
 {
     play(filePath.toLocalFile());
 }
